[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# PA Engine client library for Python

[![API Version](https://img.shields.io/badge/api-v3.16.0-blue)](https://developer.factset.com/api-catalog/pa-engine-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.PAEngine/2.2.0)](https://pypi.org/project/fds.sdk.PAEngine/v/2.2.0)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Allow clients to fetch Analytics through APIs.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 3.16.0
- SDK version: 2.2.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.protobuf.stach.extensions fds.sdk.PAEngine==2.2.0
```

### pip

```shell
pip install fds.sdk.utils fds.protobuf.stach.extensions fds.sdk.PAEngine==2.2.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.PAEngine
from fds.sdk.PAEngine.api import accounts_api
from fds.sdk.PAEngine.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.PAEngine.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.PAEngine.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.PAEngine.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = accounts_api.AccountsApi(api_client)
    path = "Client:" # str | The directory to get the accounts and sub-directories.

    try:
        # Get accounts and sub-directories in a directory
        # example passing only required values which don't have defaults set
        api_response = api_instance.get_accounts(path)

        pprint(api_response)
    except fds.sdk.PAEngine.ApiException as e:
        print("Exception when calling AccountsApi->get_accounts: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Get accounts and sub-directories in a directory
    #     api_response, http_status_code, response_headers = api_instance.get_accounts_with_http_info(path)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.PAEngine.ApiException as e:
    #     print("Exception when calling AccountsApi->get_accounts: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Get accounts and sub-directories in a directory
    #     async_result = api_instance.get_accounts_async(path)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.PAEngine.ApiException as e:
    #     print("Exception when calling AccountsApi->get_accounts: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Get accounts and sub-directories in a directory
    #     async_result = api_instance.get_accounts_with_http_info_async(path)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.PAEngine.ApiException as e:
    #     print("Exception when calling AccountsApi->get_accounts: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.PAEngine

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.PAEngine.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.PAEngine

configuration = fds.sdk.PAEngine.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.PAEngine

configuration = fds.sdk.PAEngine.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.PAEngine

configuration = fds.sdk.PAEngine.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## STACH extensions

Some methods within this client library are able to return STACH formatted data. Please see the documentation of those endpoints on how to enable and use STACH formatted data.

Documentation of the STACH extension can be found here: [stach-extensions](https://github.com/factset/stach-extensions) and here [STACH schema](https://factset.github.io/stachschema)

## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*AccountsApi* | [**get_accounts**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/AccountsApi.md#get_accounts) | **GET** /analytics/lookups/v3/accounts/{path} | Get accounts and sub-directories in a directory
*ColumnStatisticsApi* | [**get_pa_column_statistics**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ColumnStatisticsApi.md#get_pa_column_statistics) | **GET** /analytics/engines/pa/v3/columnstatistics | Get PA column statistics
*ColumnsApi* | [**get_pa_column_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ColumnsApi.md#get_pa_column_by_id) | **GET** /analytics/engines/pa/v3/columns/{id} | Get PA column settings
*ColumnsApi* | [**get_pa_columns**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ColumnsApi.md#get_pa_columns) | **GET** /analytics/engines/pa/v3/columns | Get PA columns
*ComponentsApi* | [**get_pa_component_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ComponentsApi.md#get_pa_component_by_id) | **GET** /analytics/engines/pa/v3/components/{id} | Get PA component by id
*ComponentsApi* | [**get_pa_components**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ComponentsApi.md#get_pa_components) | **GET** /analytics/engines/pa/v3/components | Get PA components
*ConfigurationsApi* | [**get_pa_commentary_configurations**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ConfigurationsApi.md#get_pa_commentary_configurations) | **GET** /analytics/engines/pa/v3/commentary/configurations | Get PA commentary configurations in a directory
*CurrenciesApi* | [**get_currencies**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CurrenciesApi.md#get_currencies) | **GET** /analytics/lookups/v3/currencies | Get currencies
*DatesApi* | [**convert_pa_dates_to_absolute_format**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/DatesApi.md#convert_pa_dates_to_absolute_format) | **GET** /analytics/engines/pa/v3/dates | Convert PA dates to absolute format
*DocumentsApi* | [**get_pa3_documents**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/DocumentsApi.md#get_pa3_documents) | **GET** /analytics/engines/pa/v3/documents/{path} | Get PA3 documents and sub-directories in a directory
*FrequenciesApi* | [**get_pa_frequencies**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/FrequenciesApi.md#get_pa_frequencies) | **GET** /analytics/engines/pa/v3/frequencies | Get PA frequencies
*GroupsApi* | [**get_pa_grouping_frequencies**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/GroupsApi.md#get_pa_grouping_frequencies) | **GET** /analytics/engines/pa/v3/grouping-frequencies | Get PA grouping frequencies
*GroupsApi* | [**get_pa_groups**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/GroupsApi.md#get_pa_groups) | **GET** /analytics/engines/pa/v3/groups | Get PA groups
*LinkedPATemplatesApi* | [**create_linked_pa_templates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplatesApi.md#create_linked_pa_templates) | **POST** /analytics/engines/pa/v3/linked-templates | Create a linked PA template
*LinkedPATemplatesApi* | [**delete_linked_pa_templates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplatesApi.md#delete_linked_pa_templates) | **DELETE** /analytics/engines/pa/v3/linked-templates/{id} | Delete a linked PA template.
*LinkedPATemplatesApi* | [**get_linked_pa_templates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplatesApi.md#get_linked_pa_templates) | **GET** /analytics/engines/pa/v3/linked-templates | Get linked PA templates
*LinkedPATemplatesApi* | [**get_linked_pa_templates_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplatesApi.md#get_linked_pa_templates_by_id) | **GET** /analytics/engines/pa/v3/linked-templates/{id} | Get linked PA template by id
*LinkedPATemplatesApi* | [**update_linked_pa_templates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplatesApi.md#update_linked_pa_templates) | **PUT** /analytics/engines/pa/v3/linked-templates/{id} | Update a linked PA template
*PACalculationsApi* | [**cancel_calculation_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#cancel_calculation_by_id) | **DELETE** /analytics/engines/pa/v3/calculations/{id} | Cancel PA calculation by id
*PACalculationsApi* | [**generate_pa_portfolio_commentary**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#generate_pa_portfolio_commentary) | **POST** /analytics/engines/pa/v3/calculations/{id}/units/{unitId}/commentary | Generate PA portfolio commentary by calculation and unit id.
*PACalculationsApi* | [**get_all_calculations**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#get_all_calculations) | **GET** /analytics/engines/pa/v3/calculations | Get all calculations
*PACalculationsApi* | [**get_calculation_parameters**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#get_calculation_parameters) | **GET** /analytics/engines/pa/v3/calculations/{id} | Get PA calculation parameters by id
*PACalculationsApi* | [**get_calculation_status_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#get_calculation_status_by_id) | **GET** /analytics/engines/pa/v3/calculations/{id}/status | Get PA calculation status by id
*PACalculationsApi* | [**get_calculation_unit_result_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#get_calculation_unit_result_by_id) | **GET** /analytics/engines/pa/v3/calculations/{id}/units/{unitId}/result | Get PA calculation result by id
*PACalculationsApi* | [**get_pa_portfolio_commentary_result_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#get_pa_portfolio_commentary_result_by_id) | **GET** /analytics/engines/pa/v3/calculations/{id}/units/{unitId}/commentary/result | Get PA Portfolio commentary result by calculation and unit id.
*PACalculationsApi* | [**get_pa_portfolio_commentary_status_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#get_pa_portfolio_commentary_status_by_id) | **GET** /analytics/engines/pa/v3/calculations/{id}/units/{unitId}/commentary/status | Get PA portfolio commentary status by calculation and unit id.
*PACalculationsApi* | [**post_and_calculate**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#post_and_calculate) | **POST** /analytics/engines/pa/v3/calculations | Create and Run PA calculation
*PACalculationsApi* | [**put_and_calculate**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationsApi.md#put_and_calculate) | **PUT** /analytics/engines/pa/v3/calculations/{id} | Create or Update PA calculation and run it.
*PricingSourcesApi* | [**get_pa_pricing_sources**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PricingSourcesApi.md#get_pa_pricing_sources) | **GET** /analytics/engines/pa/v3/pricing-sources | Get PA pricing sources
*TemplatedPAComponentsApi* | [**create_templated_pa_components**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentsApi.md#create_templated_pa_components) | **POST** /analytics/engines/pa/v3/templated-components | Create templated PA component
*TemplatedPAComponentsApi* | [**delete_templated_pa_components**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentsApi.md#delete_templated_pa_components) | **DELETE** /analytics/engines/pa/v3/templated-components/{id} | Delete templated PA component
*TemplatedPAComponentsApi* | [**get_templated_pa_component_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentsApi.md#get_templated_pa_component_by_id) | **GET** /analytics/engines/pa/v3/templated-components/{id} | Get templated PA component by id
*TemplatedPAComponentsApi* | [**get_templated_pa_components_in_path**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentsApi.md#get_templated_pa_components_in_path) | **GET** /analytics/engines/pa/v3/templated-components | Get templated PA components in path
*TemplatedPAComponentsApi* | [**update_templated_pa_components**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentsApi.md#update_templated_pa_components) | **PUT** /analytics/engines/pa/v3/templated-components/{id} | Update templated PA component
*UnlinkedPATemplatesApi* | [**create_unlinked_pa_templates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatesApi.md#create_unlinked_pa_templates) | **POST** /analytics/engines/pa/v3/unlinked-templates | Create unlinked PA template
*UnlinkedPATemplatesApi* | [**delete_unlinked_pa_templates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatesApi.md#delete_unlinked_pa_templates) | **DELETE** /analytics/engines/pa/v3/unlinked-templates/{id} | Delete unlinked PA template
*UnlinkedPATemplatesApi* | [**get_default_unlinked_pa_template_types**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatesApi.md#get_default_unlinked_pa_template_types) | **GET** /analytics/engines/pa/v3/unlinked-templates/template-types | Get default unlinked PA template types.
*UnlinkedPATemplatesApi* | [**get_details_type**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatesApi.md#get_details_type) | **GET** /analytics/engines/pa/v3/unlinked-templates/template-types/{id} | Get unlinked PA template type details by id.
*UnlinkedPATemplatesApi* | [**get_unlinked_pa_templates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatesApi.md#get_unlinked_pa_templates) | **GET** /analytics/engines/pa/v3/unlinked-templates | Get unlinked PA templates
*UnlinkedPATemplatesApi* | [**get_unlinked_pa_templates_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatesApi.md#get_unlinked_pa_templates_by_id) | **GET** /analytics/engines/pa/v3/unlinked-templates/{id} | Get unlinked PA template details by id
*UnlinkedPATemplatesApi* | [**update_unlinked_pa_templates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatesApi.md#update_unlinked_pa_templates) | **PUT** /analytics/engines/pa/v3/unlinked-templates/{id} | Update unlinked PA template


## Documentation For Models

 - [AccountDirectories](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/AccountDirectories.md)
 - [AccountDirectoriesRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/AccountDirectoriesRoot.md)
 - [CalculationMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CalculationMeta.md)
 - [CalculationStatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CalculationStatus.md)
 - [CalculationStatusMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CalculationStatusMeta.md)
 - [CalculationStatusRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CalculationStatusRoot.md)
 - [CalculationUnitStatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CalculationUnitStatus.md)
 - [CalculationUnitStatusMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CalculationUnitStatusMeta.md)
 - [CalculationsSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CalculationsSummary.md)
 - [CalculationsSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CalculationsSummaryRoot.md)
 - [ClientErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ClientErrorResponse.md)
 - [Column](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/Column.md)
 - [ColumnRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ColumnRoot.md)
 - [ColumnStatistic](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ColumnStatistic.md)
 - [ColumnStatisticRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ColumnStatisticRoot.md)
 - [ColumnSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ColumnSummary.md)
 - [ColumnSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ColumnSummaryRoot.md)
 - [ComponentSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ComponentSummary.md)
 - [ComponentSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ComponentSummaryRoot.md)
 - [Currency](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/Currency.md)
 - [CurrencyRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/CurrencyRoot.md)
 - [DateParametersSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/DateParametersSummary.md)
 - [DateParametersSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/DateParametersSummaryRoot.md)
 - [DocumentDirectories](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/DocumentDirectories.md)
 - [DocumentDirectoriesRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/DocumentDirectoriesRoot.md)
 - [Error](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/Error.md)
 - [ErrorSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ErrorSource.md)
 - [Frequency](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/Frequency.md)
 - [FrequencyRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/FrequencyRoot.md)
 - [Group](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/Group.md)
 - [GroupRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/GroupRoot.md)
 - [LinkedPATemplate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplate.md)
 - [LinkedPATemplateParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplateParameters.md)
 - [LinkedPATemplateParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplateParametersRoot.md)
 - [LinkedPATemplatePostSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplatePostSummary.md)
 - [LinkedPATemplatePostSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplatePostSummaryRoot.md)
 - [LinkedPATemplateRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplateRoot.md)
 - [LinkedPATemplateSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplateSummary.md)
 - [LinkedPATemplateSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplateSummaryRoot.md)
 - [LinkedPATemplateUpdateParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplateUpdateParameters.md)
 - [LinkedPATemplateUpdateParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/LinkedPATemplateUpdateParametersRoot.md)
 - [Metadata](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/Metadata.md)
 - [ObjectRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/ObjectRoot.md)
 - [PACalculationColumn](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationColumn.md)
 - [PACalculationDataSources](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationDataSources.md)
 - [PACalculationGroup](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationGroup.md)
 - [PACalculationParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationParameters.md)
 - [PACalculationParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationParametersRoot.md)
 - [PACalculationPricingSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACalculationPricingSource.md)
 - [PACommentary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentary.md)
 - [PACommentaryConfigurationSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryConfigurationSummary.md)
 - [PACommentaryConfigurationSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryConfigurationSummaryRoot.md)
 - [PACommentaryDate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryDate.md)
 - [PACommentaryGetStatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryGetStatus.md)
 - [PACommentaryGetStatusRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryGetStatusRoot.md)
 - [PACommentaryParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryParameters.md)
 - [PACommentaryParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryParametersRoot.md)
 - [PACommentaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryRoot.md)
 - [PACommentaryStatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryStatus.md)
 - [PACommentaryStatusRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PACommentaryStatusRoot.md)
 - [PAComponent](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PAComponent.md)
 - [PAComponentData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PAComponentData.md)
 - [PAComponentRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PAComponentRoot.md)
 - [PADateParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PADateParameters.md)
 - [PAIdentifier](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PAIdentifier.md)
 - [PAPricingSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PAPricingSource.md)
 - [PAPricingSourceRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PAPricingSourceRoot.md)
 - [PaginationInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PaginationInfo.md)
 - [PaginationMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/PaginationMeta.md)
 - [Section](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/Section.md)
 - [TemplateContentTypes](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplateContentTypes.md)
 - [TemplatedPAComponent](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponent.md)
 - [TemplatedPAComponentParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentParameters.md)
 - [TemplatedPAComponentParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentParametersRoot.md)
 - [TemplatedPAComponentPostSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentPostSummary.md)
 - [TemplatedPAComponentPostSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentPostSummaryRoot.md)
 - [TemplatedPAComponentRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentRoot.md)
 - [TemplatedPAComponentSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentSummary.md)
 - [TemplatedPAComponentSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentSummaryRoot.md)
 - [TemplatedPAComponentUpdateParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentUpdateParameters.md)
 - [TemplatedPAComponentUpdateParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/TemplatedPAComponentUpdateParametersRoot.md)
 - [UnlinkedPATemplate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplate.md)
 - [UnlinkedPATemplateCategoryAndType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateCategoryAndType.md)
 - [UnlinkedPATemplateCategoryAndTypeDetails](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateCategoryAndTypeDetails.md)
 - [UnlinkedPATemplateCategoryAndTypeDetailsRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateCategoryAndTypeDetailsRoot.md)
 - [UnlinkedPATemplateCategoryAndTypeRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateCategoryAndTypeRoot.md)
 - [UnlinkedPATemplateColumnDetails](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateColumnDetails.md)
 - [UnlinkedPATemplateGroupDetails](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateGroupDetails.md)
 - [UnlinkedPATemplateParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateParameters.md)
 - [UnlinkedPATemplateParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateParametersRoot.md)
 - [UnlinkedPATemplatePostSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatePostSummary.md)
 - [UnlinkedPATemplatePostSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplatePostSummaryRoot.md)
 - [UnlinkedPATemplateRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateRoot.md)
 - [UnlinkedPATemplateSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateSummary.md)
 - [UnlinkedPATemplateSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateSummaryRoot.md)
 - [UnlinkedPATemplateUpdateParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateUpdateParameters.md)
 - [UnlinkedPATemplateUpdateParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/PAEngine/v3/docs/UnlinkedPATemplateUpdateParametersRoot.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.PAEngine.apis and fds.sdk.PAEngine.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.PAEngine.api.default_api import DefaultApi`
- `from fds.sdk.PAEngine.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.PAEngine
from fds.sdk.PAEngine.apis import *
from fds.sdk.PAEngine.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

